/**
 * Components - Accordions
 * - 01 - Accordions
 * - 02 - Drupal Attach
 */


/*------------------------------------*\
  01 - Accordions
\*------------------------------------*/

const prototypeAccordion = (container) => {
  // Constructor
  const expandedClass = 'expanded';
  let accordions;
  let triggers;

  /**
   * Expand
   * Function used to expand elements across events.
   * @param event: Specific user event taken.
   * @see onClick
   */
  const expand = (event) => {
    const accordionPanelID = event.currentTarget.getAttribute('aria-controls');
    const accordionPanel = container.querySelector(`#${accordionPanelID}`);

    event.currentTarget.classList.add(expandedClass);
    event.currentTarget.setAttribute('aria-expanded', 'true');
    accordionPanel.classList.add(expandedClass);
    accordionPanel.removeAttribute('hidden');
  };

  /**
   * Collapse
   * Function used to collapse or hide elements across events.
   * @param event: Specific user event taken.
   * @see onClick
   */
  const collapse = (event) => {
    const accordionPanelID = event.currentTarget.getAttribute('aria-controls');
    const accordionPanel = container.querySelector(`#${accordionPanelID}`);

    event.currentTarget.classList.remove(expandedClass);
    event.currentTarget.setAttribute('aria-expanded', 'false');
    accordionPanel.classList.remove(expandedClass);
    accordionPanel.setAttribute('hidden', '');
  };

  /**
   * On Load
   * Function used to set specific attributes on elements when page is loaded.
   * @see init
   */
  const onLoad = () => {
    triggers.forEach((trigger, index) => {
      trigger.removeAttribute('tabindex');

      if (index === 0) {
        const accordionPanelID = trigger.getAttribute('aria-controls');
        const accordionPanel = container.querySelector(`#${accordionPanelID}`);

        trigger.classList.add(expandedClass);
        trigger.setAttribute('aria-expanded', 'true');
        accordionPanel.classList.add(expandedClass);
        accordionPanel.removeAttribute('hidden');
      }
    });
  };

  /**
   * On Click
   * Function used to set specific attributes on click events.
   * @param event
   * @see init
   */
  const onClick = (event) => {
    if (event.currentTarget.classList.contains(expandedClass)) {
      collapse(event);
    } else {
      expand(event);
    }
  };

  /**
   * On Keydown
   * Function used to set specific attributes on keyboard events.
   * @param event
   * @see init
   */
  const onKeydown = (event) => {
    if (event.key === 'Enter' || event.key === ' ') {
      event.preventDefault();

      if (event.currentTarget.classList.contains(expandedClass)) {
        collapse(event);
      } else {
        expand(event);
      }
    }

    if (event.key === 'ArrowDown') {
      if (event.currentTarget === triggers[triggers.length - 1]) {
        triggers[0].focus();
      } else {
        event.currentTarget.parentElement.parentElement.nextElementSibling.querySelector('.accordion__trigger').focus();
      }
    }

    if (event.key === 'ArrowUp') {
      if (event.currentTarget === triggers[0]) {
        triggers[triggers.length - 1].focus();
      } else {
        event.currentTarget.parentElement.parentElement.previousElementSibling.querySelector('.accordion__trigger').focus();
      }
    }

    if (event.key === 'Home') {
      triggers[0].focus();
    }

    if (event.key === 'End') {
      triggers[triggers.length - 1].focus();
    }
  }

  /**
   * Initialization
   * Function used to set and run any parameters, attributes and attach event listeners.
   * @param container
   */
  const init = (container) => {
    accordions = container.querySelectorAll('.accordion');
    triggers = container.querySelectorAll('.accordion__trigger');

    window.addEventListener('load', onLoad);

    triggers.forEach((trigger, index) => {
      trigger.addEventListener('click', onClick);
      trigger.addEventListener('keydown', onKeydown);
    })
  };

  // Run full program
  init(container);
};




/*------------------------------------*\
  02 - Drupal Attach
\*------------------------------------*/

Drupal.behaviors.prototypeAccordions = {
  attach(context, settings) {
    const accordions = once('prototype-accordions', context.querySelectorAll('.accordions'));

    if (accordions.length !== 0) {
      accordions.forEach((accordion) => {
        prototypeAccordion(accordion);
      });
    }
  },
};
